package com.winhc.bigdata.spark.jobs.deadbeat

import com.winhc.bigdata.spark.udf.BaseFunc
import com.winhc.bigdata.spark.utils.LoggingUtils
import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter

/**
 * @Author: XuJiakai
 * @Date: 2020/10/12 15:26
 * @Description: 失信人企业+个人 增量+全量
 */
case class dishonest_info(s: SparkSession
                          , project: String //表所在工程名
                          , table: String
                          , rowkey: String
                          , cid: String
                          , name: String
                          , card_num: String
                          , publish_date: String
                          , deleted: String
                         ) extends LoggingUtils with BaseFunc {
  @(transient@getter) val spark: SparkSession = s

  private def is_deleted(status: Int, deleted: Int): String = if ((status + deleted) == 0) "0" else "1"

   def calc(): Unit = {
    val org_tab = StringUtils.isEmpty(cid) match {
      case true => s"${table}_person"
      case false => table
    }

    val cid_f = StringUtils.isEmpty(cid) match {
      case true => ""
      case false => s",$cid as cid"
    }

    val target_table = StringUtils.isEmpty(cid) match {
      case true => s"$project.ads_deadbeat_person"
      case false => s"$project.ads_deadbeat_company"
    }

    val verify = StringUtils.isEmpty(cid) match {
      case true => s"AND      cid is not null"
      case false => s"AND     is_id_card($card_num)"
    }
    val inc_ads_table = s"$project.inc_ads_$org_tab"
    val ads_table = s"$project.ads_$org_tab"

    val intersect_cols = getColumns(inc_ads_table).intersect(getColumns(ads_table))
    val inc_ads_last_ds = getLastPartitionsOrElse(inc_ads_table, "0")
    val ads_last_ds = getLastPartitionsOrElse(ads_table, "0")
    spark.udf.register("is_deleted", is_deleted _)

    val view_fields =
      s"""
         |$rowkey as rowkey
         |${cid_f}
         |,$name as name
         |,$card_num as card_num
         |,$publish_date as publish_date
         |,$deleted as deleted
         |""".stripMargin

    def all(): Unit = {
      sql(
        s"""
           |INSERT OVERWRITE TABLE $target_table PARTITION(ds='$inc_ads_last_ds',tn='$org_tab')
           |SELECT  $view_fields
           |FROM    (
           |            SELECT  *
           |                    ,ROW_NUMBER() OVER(PARTITION BY $rowkey ORDER BY ds DESC ) AS num
           |            FROM    (
           |                        SELECT  ${intersect_cols.mkString(",")}
           |                        FROM    $ads_table
           |                        WHERE   ds = '$ads_last_ds'
           |                        UNION ALL
           |                        SELECT  ${intersect_cols.mkString(",")}
           |                        FROM    $inc_ads_table
           |                        WHERE   ds > '$ads_last_ds'
           |                    ) AS t1
           |        ) AS t2
           |WHERE   t2.num = 1
           |$verify
           |""".stripMargin)
    }

    def inc(last_ds: String): Unit = {
      sql(
        s"""
           |INSERT OVERWRITE TABLE $target_table PARTITION(ds='$inc_ads_last_ds',tn='$org_tab')
           |SELECT  $view_fields
           |FROM    $inc_ads_table
           |WHERE   ds > '$last_ds'
           |$verify
           |""".stripMargin)
    }

    val ds_cols = sql(s"show partitions $target_table").collect()
      .map(_.getString(0))
      .filter(_.contains(org_tab))
      .flatMap(_.split("/"))
      .filter(_.contains("ds"))
      .map(_.split("=")(1))
    if (ds_cols.isEmpty) {
      println("全量计算")
      all()
    } else {
      val max = ds_cols.max
      println(s"增量计算：$max")
      inc(max)
    }
  }
}
